﻿using System.Data.SqlTypes;

namespace LinqToDB.CodeModel
{
	/// <summary>
	/// This visitor wraps equality operation for some well-known types with cast to <see cref="Boolean"/> as they
	/// override equality operation to return <see cref="SqlBoolean"/> value.
	/// </summary>
	public sealed class SqlBoolEqualityConverter : ConvertCodeModelVisitor
	{
		private readonly ISet<IType> _types;

		private SqlBoolEqualityConverter(IEqualityComparer<IType> typeComparer)
		{
			_types = new HashSet<IType>(typeComparer)
			{
				WellKnownTypes.System.Data.SqlTypes.SqlBinary,
				WellKnownTypes.System.Data.SqlTypes.SqlBoolean,
				WellKnownTypes.System.Data.SqlTypes.SqlByte,
				WellKnownTypes.System.Data.SqlTypes.SqlDateTime,
				WellKnownTypes.System.Data.SqlTypes.SqlDecimal,
				WellKnownTypes.System.Data.SqlTypes.SqlDouble,
				WellKnownTypes.System.Data.SqlTypes.SqlGuid,
				WellKnownTypes.System.Data.SqlTypes.SqlInt16,
				WellKnownTypes.System.Data.SqlTypes.SqlInt32,
				WellKnownTypes.System.Data.SqlTypes.SqlInt64,
				WellKnownTypes.System.Data.SqlTypes.SqlMoney,
				WellKnownTypes.System.Data.SqlTypes.SqlSingle,
				WellKnownTypes.System.Data.SqlTypes.SqlString,

				WellKnownTypes.Microsoft.SqlServer.Types.SqlHierarchyId,
			};
		}

		/// <summary>
		/// Creates instance of converter.
		/// </summary>
		/// <param name="languageProvider">Current language provider.</param>
		/// <returns>Converter instance.</returns>
		public static ConvertCodeModelVisitor Create(ILanguageProvider languageProvider)
		{
			return new SqlBoolEqualityConverter(languageProvider.TypeEqualityComparerWithoutNRT);
		}

		protected override ICodeElement Visit(CodeBinary expression)
		{
			if (expression.Operation == BinaryOperation.Equal
				&& _types.Contains(expression.Left.Type))
				return new CodeTypeCast(WellKnownTypes.System.Boolean, expression);

			return base.Visit(expression);
		}
	}
}
